#define HUFFBITS unsigned long int
#define HTN 34
#define MXOFF   250
#include "huffarray.h"

HUFFBITS dmask = 1 << (sizeof(HUFFBITS)*8-1);

struct huffcodetab {
  char tablename[3];    /*string, containing table_description  */
  unsigned int xlen;    /*max. x-index+                 */
  unsigned int ylen;    /*max. y-index+             */
  unsigned int linbits; /*number of linbits         */
  unsigned int linmax;  /*max number to be stored in linbits    */
  int ref;      /*a positive value indicates a reference*/
  HUFFBITS *table;  /*pointer to array[xlen][ylen]      */
  unsigned char *hlen;  /*pointer to array[xlen][ylen]      */
  unsigned char (*val)[2];/*decoder tree             */
  unsigned int treelen; /*length of decoder tree        */
} ;
struct huffcodetab ht[HTN];

/***********huffman definition ends*******/
/* read the huffman decoder table */
int read_decoder_table()
{
	unsigned char tab_num_cnt=0;
	for(tab_num_cnt=0;tab_num_cnt<HTN;tab_num_cnt++)
	{
		ht[tab_num_cnt].tablename[2]='\0';
		ht[tab_num_cnt].tablename[1]=48+tab_header[tab_num_cnt][0]%10;
		ht[tab_num_cnt].tablename[0]=48+tab_header[tab_num_cnt][0]/10;
		ht[tab_num_cnt].treelen=tab_header[tab_num_cnt][1];
		ht[tab_num_cnt].xlen=tab_header[tab_num_cnt][2];
		ht[tab_num_cnt].ylen=tab_header[tab_num_cnt][3];
		ht[tab_num_cnt].linbits=tab_header[tab_num_cnt][4];
	}
	ht[0].val=v_tab_0;
	ht[1].val=v_tab_1;
	ht[2].val=v_tab_2;
	ht[3].val=v_tab_3;
	ht[4].val=v_tab_4;
	ht[5].val=v_tab_5;
	ht[6].val=v_tab_6;
	ht[7].val=v_tab_7;
	ht[8].val=v_tab_8;
	ht[9].val=v_tab_9;
	ht[10].val=v_tab_10;
	ht[11].val=v_tab_11;
	ht[12].val=v_tab_12;
	ht[13].val=v_tab_13;
	ht[14].val=v_tab_14;
	ht[15].val=v_tab_15;
	ht[16].val=v_tab_16;
	ht[17].val=v_tab_17;
	ht[18].val=v_tab_18;
	ht[19].val=v_tab_19;
	ht[20].val=v_tab_20;
	ht[21].val=v_tab_21;
	ht[22].val=v_tab_22;
	ht[23].val=v_tab_23;
	ht[24].val=v_tab_24;
	ht[25].val=v_tab_25;
	ht[26].val=v_tab_26;
	ht[27].val=v_tab_27;
	ht[28].val=v_tab_28;
	ht[29].val=v_tab_29;
	ht[30].val=v_tab_30;
	ht[31].val=v_tab_31;
	ht[32].val=v_tab_32;
	ht[33].val=v_tab_33;
  return 34;
}

/****************** huffman initial function********************/
int huffman_initialized = 0;
void initialize_huffman() {
	read_decoder_table();
}




/****************** huffman decoder function********************/


int huffman_decoder
	(bit_stream_struc *main_data_beg_bs,
	 struct huffcodetab *h,
	 int *x, int *y, int *v, int *w)
{

  HUFFBITS level;
  int point = 0;
  int error = 1;
  int bit_used_count=0;
  level     = dmask;
  //printf("1111111\n");
  if (h->val == NULL) return 2;
  //printf("tree legnth read by huffman_decoder is %d \n",h->treelen);
  /* table 0 needs no bits */
  if ( h->treelen == 0)
  {  *x = *y = 0;
     return bit_used_count;
  }


  /* Lookup in Huffman table. */

  do {
    if (h->val[point][0]==0) {   /*end of tree*/
      *x = h->val[point][1] >> 4;
      *y = h->val[point][1] & 0xf;
            //return bit_used_count;
      error = 0;

      break;
    }
    if (get1bit(main_data_beg_bs)) {
      while (h->val[point][1] >= MXOFF) point += h->val[point][1];
      point += h->val[point][1];
      bit_used_count++;
      //printf("\nDecoding Boss:main_data_beg_bs.byte_idx_ptr is %d\n",main_data_beg_bs->byte_idx_ptr-data_header);
      //printf("\nDecoding Boss:main_data_beg_bs.bit_idx %d\n",main_data_beg_bs->bit_idx);
    }
    else {
      while (h->val[point][0] >= MXOFF) point += h->val[point][0];
      point += h->val[point][0];
      bit_used_count++;
    }
    level >>= 1;
  } while (level  || (point < ht->treelen) );
  /* Check for error. */
  if (error) { /* set x and y to a medium value as a simple concealment */
    printf("Illegal Huffman code in data.\n");
    *x = (h->xlen-1 << 1);
    *y = (h->ylen-1 << 1);
  }
  /* Process sign encodings for quadruples tables. */

  if (h->tablename[0] == '3'
      && (h->tablename[1] == '2' || h->tablename[1] == '3')) {
     *v = (*y>>3) & 1;
     *w = (*y>>2) & 1;
     *x = (*y>>1) & 1;
     *y = *y & 1;

     /* v, w, x and y are reversed in the bitstream.
        switch them around to make test bistream work. */

/*   {int i=*v; *v=*y; *y=i; i=*w; *w=*x; *x=i;}  MI */

     if (*v)
     {
    	bit_used_count++;
    	if (get1bit(main_data_beg_bs) == 1) *v = -*v;
     }

     if (*w)
     {
    	bit_used_count++;
    	if (get1bit(main_data_beg_bs) == 1) *w = -*w;
     }
     if (*x)
     {
    	bit_used_count++;
    	if (get1bit(main_data_beg_bs) == 1) *x = -*x;
     }
     if (*y)
     {
    	bit_used_count++;
    	if (get1bit(main_data_beg_bs) == 1) *y = -*y;
     }

     return bit_used_count;
     }

  /* Process sign and escape encodings for dual tables. */

  else {

      /* x and y are reversed in the test bitstream.
         Reverse x and y here to make test bitstream work. */

/*    removed 11/11/92 -ag
        {int i=*x; *x=*y; *y=i;}
*///  printf("22221\n");
     if (h->linbits)
       if ((h->xlen-1) == *x)
         {*x += getbits(main_data_beg_bs,h->linbits); bit_used_count+=h->linbits;}
     if (*x)
     {
    	bit_used_count++;
    	if (get1bit(main_data_beg_bs) == 1) *x = -*x;
     }
     if (h->linbits)
       if ((h->ylen-1) == *y)
         {*y += getbits(main_data_beg_bs,h->linbits);  bit_used_count+=h->linbits;}
     if (*y)
     {
    	bit_used_count++;
    	if (get1bit(main_data_beg_bs) == 1) *y = -*y;
     }
     return bit_used_count;
     }
	 //main_data_beg_bs->byte_idx_ptr=byte_idx_ptr_temp_2;
     //main_data_beg_bs->bit_idx=bit_idx_temp_2;
  return error;
}




/*********************** mp3_Huffman decoder****************/
void huffman_decode
	(long int is[SBLIMIT][SSLIMIT],
	 side_info_type *si,
	 int ch, int gr,
	 header_info_type *header_info,
	 bit_stream_struc *main_data_beg_bs)
{
   unsigned char *byte_idx_ptr_temp;
   unsigned char bit_idx_temp;

   byte_idx_ptr_temp = main_data_beg_bs->byte_idx_ptr;
   bit_idx_temp=main_data_beg_bs->bit_idx;

   int i, x=1, y=1;
   int v, w;
   int bit_used=0;
   struct huffcodetab *h;
   h=malloc(sizeof(struct huffcodetab));
   h=(struct huffcodetab *)h;

   int region1Start;
   int region2Start;
   int samplefreq;
   int currentBit, grBits;
   int part2_length;
   struct gr_info_type *gi;
   //int bt = (*si).ch[ch].gr[gr].window_switching_flag && ((*si).ch[ch].gr[gr].block_type == 2);

   gi = (struct gr_info_type *) &(*si).ch[ch].gr[gr];
   samplefreq = header_info->sampling_frequency_index + (header_info->ID * 3);
/*calculate part2_length*/

	  if (/*gi->window_switching_flag &&*/ (gi->block_type == 2)) {
		 if (gi->mixed_block_flag)
         {
        	 part2_length=17*slen[0][gi->scalefac_compress]
        	             +18*slen[1][gi->scalefac_compress];

          }
          else {
         	 part2_length=18*slen[0][gi->scalefac_compress]
         	             +18*slen[1][gi->scalefac_compress];
         }
       }
       else {   /* LONG types 0,1,3 */
			  part2_length=11*slen[0][gi->scalefac_compress]
			              +10*slen[1][gi->scalefac_compress];

       }


//printf("The following is is serving as within one granual:\n");
  // printf("part2_length is %d\n",part2_length);
   /*printf("main_data_beg_bs.byte_idx_ptr is %d\n",main_data_beg_bs->byte_idx_ptr-data_header);
   printf("main_data_beg_bs.bit_idx %d\n",main_data_beg_bs->bit_idx);*/

   /*if((ch==1)&&(gr==1)) main_data_beg_bs->byte_idx_ptr=main_data_beg_bs->byte_idx_ptr+part2_length/8-1;
   else*/ main_data_beg_bs->byte_idx_ptr=main_data_beg_bs->byte_idx_ptr+part2_length/8;
   if(( main_data_beg_bs->bit_idx <  (part2_length)%8))
   {

	   main_data_beg_bs->bit_idx=8-(part2_length)%8+main_data_beg_bs->bit_idx;
	   main_data_beg_bs->byte_idx_ptr++;
   }
   else
   main_data_beg_bs->bit_idx=main_data_beg_bs->bit_idx-((part2_length)%8);
  /*printf("---------huffman coded data address for gr[%d],ch[%d] is as follows--------\n",gr,ch);
	printf("row is %d,   ",(main_data_beg_bs->byte_idx_ptr-data_header)/39);
	printf("column is %d.\n ",(main_data_beg_bs->byte_idx_ptr-data_header)%39);
    printf("main_data_beg_bs.byte_idx_ptr is %d    ",main_data_beg_bs->byte_idx_ptr-data_header);
    printf("main_data_beg_bs.bit_idx %d\n",main_data_beg_bs->bit_idx);
    printf("---------huffman coded data address for gr[%d],ch[%d] Ends--------\n",gr,ch);*/

   /* Find region boundary for short block case. */

   if ( ((*si).ch[ch].gr[gr].window_switching_flag) &&
        ((*si).ch[ch].gr[gr].block_type == 2) ) {

      /* Region2. */
        region1Start = 36;  /* sfb[9/3]*3=36 */
        region2Start = 576; /* No Region2 for short block case. */
   }
   else {          /* Find region boundary for long block case. */

      region1Start = sfBandIndex[samplefreq]
                           .l[(*si).ch[ch].gr[gr].region0_count + 1]; /* MI */
      region2Start = sfBandIndex[samplefreq]
                              .l[(*si).ch[ch].gr[gr].region0_count +
                              (*si).ch[ch].gr[gr].region1_count + 2]; /* MI */
      }

   /* Read bigvalues area. */
  //    printf("big value        is %d",(*si).ch[ch].gr[gr].big_values*2);


   for (i=0; i<(*si).ch[ch].gr[gr].big_values*2; i+=2) {

      if      (i<region1Start) h = &ht[(*si).ch[ch].gr[gr].table_select[0]];
      else if (i<region2Start) h = &ht[(*si).ch[ch].gr[gr].table_select[1]];
           else                h = &ht[(*si).ch[ch].gr[gr].table_select[2]];
      //printf("tree legnth inside the fucntion III_huffman_decode is %d \n\n",h->treelen);
      bit_used+=huffman_decoder(main_data_beg_bs,h, &x, &y, &v, &w);
      is[i/SSLIMIT][i%SSLIMIT] = x;
      is[(i+1)/SSLIMIT][(i+1)%SSLIMIT] = y;
      }
   /* Read count1 area. */

   h = &ht[((*si).ch[ch].gr[gr].count1table_select)+32];
   //printf("count1table_select is %d\n",((*si).ch[ch].gr[gr].count1table_select)+32);
   while ((bit_used <  ((*si).ch[ch].gr[gr].part2_3_length - part2_length)) &&
     ( i < SSLIMIT*SBLIMIT )) {

      bit_used+=huffman_decoder(main_data_beg_bs,h, &x, &y, &v, &w);
      is[i/SSLIMIT][i%SSLIMIT] = v;
      is[(i+1)/SSLIMIT][(i+1)%SSLIMIT] = w;
      is[(i+2)/SSLIMIT][(i+2)%SSLIMIT] = x;
      is[(i+3)/SSLIMIT][(i+3)%SSLIMIT] = y;
      i += 4;
      }
   /* Zero out rest. */
   for (; i<SSLIMIT*SBLIMIT; i++)
      is[i/SSLIMIT][i%SSLIMIT] = 0;
 /*  printf("the 576 sampling value is the following:\n");
   for(x=0;x<SBLIMIT;x++)
   {	printf("subband %d :",x);
        for(y=0;y<SSLIMIT;y++)
        printf("%d ",is[x][y]);

        printf("\n");


    }*/


   main_data_beg_bs->byte_idx_ptr=byte_idx_ptr_temp;
   main_data_beg_bs->bit_idx=bit_idx_temp;
}





/***********huffman definition ends*******/


